# Unit tests for presentation_manager.py methods.
#
# Copyright 2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=protected-access
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-statements

"""Unit tests for presentation_manager.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

if TYPE_CHECKING:
    from unittest.mock import MagicMock

    from .orca_test_context import OrcaTestContext


@pytest.mark.unit
class TestPresentationManager:
    """Test PresentationManager class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Returns dependencies for presentation_manager module testing."""

        additional_modules = [
            "orca.braille_presenter",
            "orca.live_region_presenter",
            "orca.sound_presenter",
            "orca.speech_manager",
            "orca.speech_presenter",
            "orca.typing_echo_presenter",
        ]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_message = test_context.Mock()
        debug_mock.print_tokens = test_context.Mock()
        debug_mock.print_exception = test_context.Mock()
        debug_mock.debugFile = None
        debug_mock.LEVEL_INFO = 800
        debug_mock.LEVEL_WARNING = 600

        braille_mock = essential_modules["orca.braille"]
        braille_mock.killFlash = test_context.Mock()
        braille_mock.displayMessage = test_context.Mock()
        braille_mock.clear = test_context.Mock()
        mock_line = test_context.Mock()
        mock_line.regions = []
        mock_line.add_regions = test_context.Mock(side_effect=mock_line.regions.append)
        braille_mock.Line = test_context.Mock(return_value=mock_line)
        braille_mock.add_line = test_context.Mock()
        braille_mock.setFocus = test_context.Mock()
        braille_mock.refresh = test_context.Mock()

        braille_presenter_mock = essential_modules["orca.braille_presenter"]
        braille_presenter_instance = test_context.Mock()
        braille_presenter_instance.use_braille = test_context.Mock(return_value=True)
        braille_presenter_instance.get_flash_messages_are_enabled = test_context.Mock(
            return_value=True,
        )
        braille_presenter_instance.get_flash_messages_are_detailed = test_context.Mock(
            return_value=True,
        )
        braille_presenter_instance.get_flashtime_from_settings = test_context.Mock(
            return_value=5000,
        )
        braille_presenter_instance.kill_flash = test_context.Mock()
        braille_presenter_instance.present_message = test_context.Mock()
        braille_presenter_instance.present_regions = test_context.Mock()
        braille_presenter_mock.get_presenter = test_context.Mock(
            return_value=braille_presenter_instance,
        )

        live_region_presenter_mock = essential_modules["orca.live_region_presenter"]
        live_region_instance = test_context.Mock()
        live_region_instance.flush_messages = test_context.Mock()
        live_region_presenter_mock.get_presenter = test_context.Mock(
            return_value=live_region_instance,
        )

        sound_presenter_mock = essential_modules["orca.sound_presenter"]
        sound_presenter_instance = test_context.Mock()
        sound_presenter_instance.play = test_context.Mock()
        sound_presenter_mock.get_presenter = test_context.Mock(
            return_value=sound_presenter_instance,
        )

        speech_manager_mock = essential_modules["orca.speech_manager"]
        speech_manager_instance = test_context.Mock()
        speech_manager_instance.interrupt_speech = test_context.Mock()
        speech_manager_instance.start_speech = test_context.Mock()
        speech_manager_instance.shutdown_speech = test_context.Mock()
        speech_manager_instance.refresh_speech = test_context.Mock()
        speech_manager_instance.get_speech_is_enabled_and_not_muted = test_context.Mock(
            return_value=True,
        )
        speech_manager_instance.get_speech_is_muted = test_context.Mock(return_value=False)
        speech_manager_mock.get_manager = test_context.Mock(return_value=speech_manager_instance)

        speech_presenter_mock = essential_modules["orca.speech_presenter"]
        speech_presenter_instance = test_context.Mock()
        speech_presenter_instance.present_key_event = test_context.Mock()
        speech_presenter_instance.speak_message = test_context.Mock()
        speech_presenter_instance.speak_contents = test_context.Mock()
        speech_presenter_instance.speak_character = test_context.Mock()
        speech_presenter_instance.spell_item = test_context.Mock()
        speech_presenter_instance.spell_phonetically = test_context.Mock()
        speech_presenter_instance.get_messages_are_detailed = test_context.Mock(return_value=True)
        speech_presenter_mock.get_presenter = test_context.Mock(
            return_value=speech_presenter_instance,
        )

        typing_echo_presenter_mock = essential_modules["orca.typing_echo_presenter"]
        typing_echo_instance = test_context.Mock()
        typing_echo_instance.echo_keyboard_event = test_context.Mock()
        typing_echo_presenter_mock.get_presenter = test_context.Mock(
            return_value=typing_echo_instance,
        )

        script_manager_mock = essential_modules["orca.script_manager"]
        mock_script = test_context.Mock()
        speech_gen = test_context.Mock()
        speech_gen.voice = test_context.Mock(return_value=[{"family": "default"}])
        speech_gen.generate_contents = test_context.Mock(return_value=["generated speech"])
        mock_script.get_speech_generator = test_context.Mock(return_value=speech_gen)

        braille_gen = test_context.Mock()
        braille_gen.generate_contents = test_context.Mock(
            return_value=([test_context.Mock()], test_context.Mock()),
        )
        mock_script.get_braille_generator = test_context.Mock(return_value=braille_gen)
        script_manager_instance = test_context.Mock()
        script_manager_instance.get_active_script = test_context.Mock(return_value=mock_script)
        script_manager_mock.get_manager = test_context.Mock(return_value=script_manager_instance)

        return essential_modules

    def test_get_manager_returns_singleton(self, test_context: OrcaTestContext) -> None:
        """Test get_manager returns the same instance."""

        self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager1 = get_manager()
        manager2 = get_manager()

        assert manager1 is manager2
        assert manager1.__class__.__name__ == "PresentationManager"

    def test_interrupt_presentation(self, test_context: OrcaTestContext) -> None:
        """Test interrupt_presentation interrupts speech and braille."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.interrupt_presentation()

        speech_manager = essential_modules["orca.speech_manager"].get_manager()
        speech_manager.interrupt_speech.assert_called_once()
        braille_presenter = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter.kill_flash.assert_called_once()
        live_region = essential_modules["orca.live_region_presenter"].get_presenter()
        live_region.flush_messages.assert_called_once()

    def test_interrupt_presentation_without_kill_flash(self, test_context: OrcaTestContext) -> None:
        """Test interrupt_presentation with kill_flash=False."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.interrupt_presentation(kill_flash=False)

        speech_manager = essential_modules["orca.speech_manager"].get_manager()
        speech_manager.interrupt_speech.assert_called_once()
        essential_modules["orca.braille"].killFlash.assert_not_called()

    def test_present_keyboard_event(self, test_context: OrcaTestContext) -> None:
        """Test present_keyboard_event delegates to typing_echo_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_event = test_context.Mock()

        manager.present_keyboard_event(mock_event)

        typing_echo = essential_modules["orca.typing_echo_presenter"].get_presenter()
        typing_echo.echo_keyboard_event.assert_called_once_with(mock_event)

    def test_present_key_event_delegates(self, test_context: OrcaTestContext) -> None:
        """Test present_key_event delegates to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_event = test_context.Mock()
        mock_event.get_key_name.return_value = "BackSpace"
        manager.present_key_event(mock_event)

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.present_key_event.assert_called_once_with(mock_event)

    def test_present_message_delegates_speech(self, test_context: OrcaTestContext) -> None:
        """Test present_message delegates speech to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.present_message("This is a full message")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_message.assert_called_once_with(
            "This is a full message",
        )
        braille_presenter = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter.present_message.assert_called()

    def test_present_message_with_brief(self, test_context: OrcaTestContext) -> None:
        """Test present_message uses full when messages_are_detailed is True."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.present_message("Full message", brief="Brief")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_message.assert_called_once_with(
            "Full message",
        )

    def test_present_message_empty_string(self, test_context: OrcaTestContext) -> None:
        """Test present_message with empty string returns early."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.present_message("")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_message.assert_not_called()
        essential_modules["orca.braille"].displayMessage.assert_not_called()

    def test_present_message_braille_disabled(self, test_context: OrcaTestContext) -> None:
        """Test present_message when braille is disabled."""

        essential_modules = self._setup_dependencies(test_context)
        braille_presenter = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter.use_braille.return_value = False

        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.present_message("Test message")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_message.assert_called()
        essential_modules["orca.braille"].displayMessage.assert_not_called()

    def test_present_message_uses_brief(self, test_context: OrcaTestContext) -> None:
        """Test present_message uses brief when messages_are_detailed is False."""

        essential_modules = self._setup_dependencies(test_context)
        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.get_messages_are_detailed.return_value = False

        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.present_message("Full message", brief="Brief")

        speech_pres.speak_message.assert_called_once_with(
            "Brief",
        )

    def test_present_message_speech_disabled(self, test_context: OrcaTestContext) -> None:
        """Test present_message skips speech when speech is disabled."""

        essential_modules = self._setup_dependencies(test_context)
        speech_mgr = essential_modules["orca.speech_manager"].get_manager()
        speech_mgr.get_speech_is_enabled_and_not_muted.return_value = False

        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.present_message("Test message")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_message.assert_not_called()

    def test_speak_message_muted(self, test_context: OrcaTestContext) -> None:
        """Test speak_message skips speech when muted."""

        essential_modules = self._setup_dependencies(test_context)
        speech_mgr = essential_modules["orca.speech_manager"].get_manager()
        speech_mgr.get_speech_is_muted.return_value = True

        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.speak_message("Hello world")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_message.assert_not_called()

    def test_speak_accessible_text_muted(self, test_context: OrcaTestContext) -> None:
        """Test speak_accessible_text skips speech when muted."""

        essential_modules = self._setup_dependencies(test_context)
        speech_mgr = essential_modules["orca.speech_manager"].get_manager()
        speech_mgr.get_speech_is_muted.return_value = True

        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_obj = test_context.Mock()
        manager.speak_accessible_text(mock_obj, "Hello")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_accessible_text.assert_not_called()

    def test_play_sound(self, test_context: OrcaTestContext) -> None:
        """Test play_sound delegates to sound_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import PresentationManager

        mock_sound = test_context.Mock()
        PresentationManager.play_sound(mock_sound)

        sound_presenter = essential_modules["orca.sound_presenter"].get_presenter()
        sound_presenter.play.assert_called_once_with(mock_sound, True)

    def test_play_sound_no_interrupt(self, test_context: OrcaTestContext) -> None:
        """Test play_sound with interrupt=False."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import PresentationManager

        mock_sound = test_context.Mock()
        PresentationManager.play_sound(mock_sound, interrupt=False)

        sound_presenter = essential_modules["orca.sound_presenter"].get_presenter()
        sound_presenter.play.assert_called_once_with(mock_sound, False)

    def test_present_braille_message(self, test_context: OrcaTestContext) -> None:
        """Test present_braille_message shows braille message."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import PresentationManager

        PresentationManager.present_braille_message("Test braille", restore_previous=False)

        braille_presenter = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter.present_message.assert_called_once_with(
            "Test braille",
            restore_previous=False,
        )

    def test_present_braille_message_braille_disabled(self, test_context: OrcaTestContext) -> None:
        """Test present_braille_message delegates even when braille is disabled."""

        essential_modules = self._setup_dependencies(test_context)
        braille_presenter_instance = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter_instance.use_braille.return_value = False

        from orca.presentation_manager import PresentationManager

        PresentationManager.present_braille_message("Test braille")

        braille_presenter_instance.present_message.assert_called_once()

    def test_spell_item_delegates(self, test_context: OrcaTestContext) -> None:
        """Test spell_item delegates to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.spell_item("abc")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.spell_item.assert_called_once_with("abc")

    def test_spell_phonetically_delegates(self, test_context: OrcaTestContext) -> None:
        """Test spell_phonetically delegates to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.spell_phonetically("ab")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.spell_phonetically.assert_called_once_with("ab")

    def test_speak_character_delegates(self, test_context: OrcaTestContext) -> None:
        """Test speak_character delegates to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.speak_character("a")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_character.assert_called_once_with(
            "a",
            voice_from="a",
            cap_style=None,
            obj=None,
        )

    def test_speak_message_delegates(self, test_context: OrcaTestContext) -> None:
        """Test speak_message delegates to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.speak_message("Hello world")

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_message.assert_called_once_with(
            "Hello world",
        )

    def test_speak_contents_delegates(self, test_context: OrcaTestContext) -> None:
        """Test speak_contents delegates to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_contents = [(test_context.Mock(), 0, 10, "test text")]
        manager.speak_contents(mock_contents)

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_contents.assert_called_once_with(mock_contents)

    def test_speak_contents_with_kwargs_delegates(self, test_context: OrcaTestContext) -> None:
        """Test speak_contents passes keyword arguments to speech_presenter."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_contents = [(test_context.Mock(), 0, 10, "test text")]
        mock_prior = test_context.Mock()
        manager.speak_contents(mock_contents, priorObj=mock_prior)

        speech_pres = essential_modules["orca.speech_presenter"].get_presenter()
        speech_pres.speak_contents.assert_called_once_with(mock_contents, priorObj=mock_prior)

    def test_display_contents(self, test_context: OrcaTestContext) -> None:
        """Test display_contents delegates to braille presenter."""

        essential_modules = self._setup_dependencies(test_context)

        script_manager = essential_modules["orca.script_manager"].get_manager()
        script = script_manager.get_active_script()

        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_contents = [(test_context.Mock(), 0, 10, "test text")]
        manager.display_contents(mock_contents)

        braille_presenter_instance = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter_instance.display_generated_contents.assert_called_once_with(
            script,
            mock_contents,
        )

    def test_display_contents_braille_disabled(self, test_context: OrcaTestContext) -> None:
        """Test display_contents still delegates to braille presenter (which checks internally)."""

        essential_modules = self._setup_dependencies(test_context)

        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_contents = [(test_context.Mock(), 0, 10, "test text")]
        manager.display_contents(mock_contents)

        braille_presenter_instance = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter_instance.display_generated_contents.assert_called_once()

    def test_display_contents_no_active_script(self, test_context: OrcaTestContext) -> None:
        """Test display_contents returns early when no active script."""

        essential_modules = self._setup_dependencies(test_context)
        script_manager = essential_modules["orca.script_manager"].get_manager()
        script_manager.get_active_script.return_value = None

        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_contents = [(test_context.Mock(), 0, 10, "test text")]
        manager.display_contents(mock_contents)

        braille_presenter_instance = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter_instance.display_generated_contents.assert_not_called()

    def test_display_contents_empty_regions(self, test_context: OrcaTestContext) -> None:
        """Test display_contents delegates even with empty contents."""

        essential_modules = self._setup_dependencies(test_context)

        from orca.presentation_manager import get_manager

        manager = get_manager()
        mock_contents: list = []
        manager.display_contents(mock_contents)

        braille_presenter_instance = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter_instance.display_generated_contents.assert_called_once()

    @pytest.mark.parametrize(
        "flash_enabled,use_braille,expected_flash_called",
        [
            pytest.param(True, True, True, id="flash_and_braille_enabled"),
            pytest.param(False, True, False, id="flash_disabled"),
            pytest.param(True, False, False, id="braille_disabled"),
            pytest.param(False, False, False, id="both_disabled"),
        ],
    )
    def test_present_message_braille_conditions(
        self,
        test_context: OrcaTestContext,
        flash_enabled: bool,
        use_braille: bool,
        expected_flash_called: bool,
    ) -> None:
        """Test present_message braille behavior under various conditions."""

        essential_modules = self._setup_dependencies(test_context)
        braille_presenter_instance = essential_modules["orca.braille_presenter"].get_presenter()
        braille_presenter_instance.use_braille.return_value = use_braille
        braille_presenter_instance.get_flash_messages_are_enabled.return_value = flash_enabled

        from orca.presentation_manager import get_manager

        manager = get_manager()
        manager.present_message("Test message")

        if expected_flash_called:
            braille_presenter_instance.present_message.assert_called()
        else:
            braille_presenter_instance.present_message.assert_not_called()



# Unit tests for keybindings.py methods.
#
# Copyright 2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=import-outside-toplevel
# pylint: disable=protected-access

"""Unit tests for keybindings.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

if TYPE_CHECKING:
    from .orca_test_context import OrcaTestContext


def _setup_keybindings_dependencies(test_context: OrcaTestContext) -> None:
    """Mock only the dependencies keybindings.py needs, not keybindings itself."""

    i18n_mock = test_context.Mock()
    i18n_mock._ = lambda x: x
    i18n_mock.C_ = lambda c, x: x
    test_context.patch_module("orca.orca_i18n", i18n_mock)

    input_event_manager_mock = test_context.Mock()
    input_event_manager_mock.get_manager = test_context.Mock(
        return_value=test_context.Mock(),
    )
    test_context.patch_module("orca.input_event_manager", input_event_manager_mock)


@pytest.mark.unit
class TestAltGrModifierSupport:
    """Test AltGr (ISO_Level3_Shift) modifier support in keybindings."""

    @staticmethod
    def _create_binding(
        keysymstring: str,
        modifiers: int,
        keyval: int = 0,
        keycode: int = 0,
    ):
        """Creates a KeyBinding with pre-populated keyval/keycode to avoid Gdk.Keymap."""

        from orca.keybindings import KeyBinding

        kb = KeyBinding(keysymstring, modifiers)
        kb.keyval = keyval
        kb.keycode = keycode
        return kb

    def test_extra_altgr_prevents_match(self, test_context: OrcaTestContext) -> None:
        """Test that AltGr in event modifiers prevents matching a non-AltGr binding."""

        _setup_keybindings_dependencies(test_context)
        from orca.keybindings import ALTGR_MODIFIER_MASK, ORCA_MODIFIER_MASK

        kb = self._create_binding("z", ORCA_MODIFIER_MASK, keyval=122, keycode=52)
        assert kb.matches(122, 52, ORCA_MODIFIER_MASK | ALTGR_MODIFIER_MASK) is False

    def test_altgr_keycode_fallback(self, test_context: OrcaTestContext) -> None:
        """Test keycode fallback when AltGr changes the keyval."""

        _setup_keybindings_dependencies(test_context)
        from orca.keybindings import ALTGR_MODIFIER_MASK, ORCA_MODIFIER_MASK

        orca_altgr = ORCA_MODIFIER_MASK | ALTGR_MODIFIER_MASK
        kb = self._create_binding("period", orca_altgr, keyval=46, keycode=60)
        assert kb.matches(183, 60, orca_altgr) is True

    def test_orca_modifier_enables_keycode_fallback(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test that Orca modifier enables keycode fallback for AZERTY-like layouts."""

        _setup_keybindings_dependencies(test_context)
        from orca.keybindings import ORCA_MODIFIER_MASK

        kb = self._create_binding("9", ORCA_MODIFIER_MASK, keyval=57, keycode=18)
        assert kb.matches(231, 18, ORCA_MODIFIER_MASK) is True



# Unit tests for braille_presenter.py methods.
#
# Copyright 2025-2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=protected-access
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-lines

"""Unit tests for braille_presenter.py methods."""

from __future__ import annotations

import unittest.mock
from typing import TYPE_CHECKING

import gi

gi.require_version("Gtk", "3.0")

import pytest

if TYPE_CHECKING:
    from unittest.mock import MagicMock

    from .orca_test_context import OrcaTestContext


@pytest.mark.unit
class TestBraillePresenter:
    """Test BraillePresenter methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Set up mocks for braille_presenter dependencies."""

        additional_modules = ["orca.braille", "orca.braille_monitor", "orca.orca_platform"]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        platform_mock = essential_modules["orca.orca_platform"]
        platform_mock.tablesdir = "/usr/share/liblouis/tables"

        from orca import gsettings_registry

        registry = gsettings_registry.get_registry()
        registry.clear_runtime_values()

        test_context.patch(
            "orca.braille_presenter.BraillePresenter._get_table_files",
            return_value=[
                "en-us-g1.ctb",
                "en-us-g2.ctb",
                "en-us-comp8.ctb",
                "fr-bfu-g2.ctb",
                "de-g2.ctb",
            ],
        )

        return essential_modules

    def test_get_braille_is_enabled(self, test_context: OrcaTestContext):
        """Test getting braille enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_braille_is_enabled() is True

        presenter.set_braille_is_enabled(False)
        assert presenter.get_braille_is_enabled() is False

    def test_set_braille_is_enabled(self, test_context: OrcaTestContext):
        """Test setting braille enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_braille_is_enabled(True)
        assert result is True
        assert presenter.get_braille_is_enabled() is True

        result = presenter.set_braille_is_enabled(False)
        assert result is True
        assert presenter.get_braille_is_enabled() is False

    def test_get_contraction_table(self, test_context: OrcaTestContext):
        """Test getting contraction table."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_contraction_table() == ""

        presenter.set_contraction_table("en-us-g2")
        assert presenter.get_contraction_table() == "en-us-g2"

    def test_get_available_contraction_tables(self, test_context: OrcaTestContext):
        """Test getting available contraction tables."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        tables = presenter.get_available_contraction_tables()
        expected = ["en-us-g1", "en-us-g2", "en-us-comp8", "fr-bfu-g2", "de-g2"]
        assert tables == expected

    def test_set_contraction_table_valid(self, test_context: OrcaTestContext):
        """Test setting valid contraction table."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_contraction_table("en-us-g2")
        assert result is True
        assert presenter.get_contraction_table_path() == "/usr/share/liblouis/tables/en-us-g2.ctb"

        result = presenter.set_contraction_table("en-us-g1.ctb")
        assert result is True
        assert presenter.get_contraction_table_path() == "/usr/share/liblouis/tables/en-us-g1.ctb"

    def test_set_contraction_table_invalid(self, test_context: OrcaTestContext):
        """Test setting invalid contraction table."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_contraction_table("nonexistent")
        assert result is False

        result = presenter.set_contraction_table("nonexistent.ctb")
        assert result is False

    def test_set_contraction_table_empty_string(self, test_context: OrcaTestContext):
        """Test setting empty contraction table returns False."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_contraction_table("")
        assert result is False

    def test_get_indicator_styles(self, test_context: OrcaTestContext):
        """Test getting indicator styles."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_selector_indicator() == "dots78"
        assert presenter.get_link_indicator() == "dots78"
        assert presenter.get_text_attributes_indicator() == "none"

        presenter.set_link_indicator("dot7")
        assert presenter.get_link_indicator() == "dot7"

    def test_set_indicator_styles(self, test_context: OrcaTestContext):
        """Test setting indicator styles."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_selector_indicator("dots78")
        assert result is True
        assert presenter.get_selector_indicator() == "dots78"

        result = presenter.set_link_indicator("dot8")
        assert result is True
        assert presenter.get_link_indicator() == "dot8"

        result = presenter.set_text_attributes_indicator("none")
        assert result is True
        assert presenter.get_text_attributes_indicator() == "none"

        result = presenter.set_selector_indicator("invalid")
        assert result is False

        result = presenter.set_link_indicator("invalid")
        assert result is False

        result = presenter.set_text_attributes_indicator("invalid")
        assert result is False

    def test_end_of_line_indicator(self, test_context: OrcaTestContext):
        """Test end of line indicator settings."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_end_of_line_indicator_is_enabled() is True

        result = presenter.set_end_of_line_indicator_is_enabled(False)
        assert result is True
        assert presenter.get_end_of_line_indicator_is_enabled() is False

        result = presenter.set_end_of_line_indicator_is_enabled(True)
        assert result is True
        assert presenter.get_end_of_line_indicator_is_enabled() is True

    def test_use_braille(self, test_context: OrcaTestContext):
        """Test use_braille method."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        presenter.set_monitor_is_enabled(False)
        assert presenter.use_braille() is True

        presenter.set_braille_is_enabled(False)
        presenter.set_monitor_is_enabled(True)
        assert presenter.use_braille() is True

        presenter.set_monitor_is_enabled(False)
        assert presenter.use_braille() is False

    def test_use_verbose_braille(self, test_context: OrcaTestContext):
        """Test use_verbose_braille method."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.use_verbose_braille() is True

        presenter.set_verbosity_level("brief")
        assert presenter.use_verbose_braille() is False

    def test_get_verbosity_level(self, test_context: OrcaTestContext):
        """Test getting verbosity level."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_verbosity_level() == "verbose"

        presenter.set_verbosity_level("brief")
        assert presenter.get_verbosity_level() == "brief"

    def test_set_verbosity_level(self, test_context: OrcaTestContext):
        """Test setting verbosity level."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_verbosity_level("brief")
        assert result is True
        assert presenter.get_verbosity_level() == "brief"

        result = presenter.set_verbosity_level("verbose")
        assert result is True
        assert presenter.get_verbosity_level() == "verbose"

        result = presenter.set_verbosity_level("invalid")
        assert result is False

    def test_use_full_rolenames(self, test_context: OrcaTestContext):
        """Test use_full_rolenames method."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.use_full_rolenames() is True

        presenter.set_rolename_style("brief")
        assert presenter.use_full_rolenames() is False

    def test_get_rolename_style(self, test_context: OrcaTestContext):
        """Test getting rolename style."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_rolename_style() == "verbose"

        presenter.set_rolename_style("brief")
        assert presenter.get_rolename_style() == "brief"

    def test_set_rolename_style(self, test_context: OrcaTestContext):
        """Test setting rolename style."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_rolename_style("brief")
        assert result is True
        assert presenter.get_rolename_style() == "brief"

        result = presenter.set_rolename_style("verbose")
        assert result is True
        assert presenter.get_rolename_style() == "verbose"

        result = presenter.set_rolename_style("invalid")
        assert result is False

    def test_get_display_ancestors(self, test_context: OrcaTestContext):
        """Test getting display ancestors setting."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_display_ancestors() is True

        presenter.set_display_ancestors(False)
        assert presenter.get_display_ancestors() is False

    def test_set_display_ancestors(self, test_context: OrcaTestContext):
        """Test setting display ancestors."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_display_ancestors(True)
        assert result is True
        assert presenter.get_display_ancestors() is True

        result = presenter.set_display_ancestors(False)
        assert result is True
        assert presenter.get_display_ancestors() is False

    def test_get_contracted_braille_is_enabled(self, test_context: OrcaTestContext):
        """Test getting contracted braille enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_contracted_braille_is_enabled() is False

        presenter.set_contracted_braille_is_enabled(True)
        assert presenter.get_contracted_braille_is_enabled() is True

    def test_set_contracted_braille_is_enabled(self, test_context: OrcaTestContext):
        """Test setting contracted braille enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_contracted_braille_is_enabled(True)
        assert result is True
        assert presenter.get_contracted_braille_is_enabled() is True

        result = presenter.set_contracted_braille_is_enabled(False)
        assert result is True
        assert presenter.get_contracted_braille_is_enabled() is False

    def test_get_computer_braille_at_cursor_is_enabled(self, test_context: OrcaTestContext):
        """Test getting computer braille at cursor enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_computer_braille_at_cursor_is_enabled() is True

        presenter.set_computer_braille_at_cursor_is_enabled(False)
        assert presenter.get_computer_braille_at_cursor_is_enabled() is False

    def test_set_computer_braille_at_cursor_is_enabled(self, test_context: OrcaTestContext):
        """Test setting computer braille at cursor enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_computer_braille_at_cursor_is_enabled(True)
        assert result is True
        assert presenter.get_computer_braille_at_cursor_is_enabled() is True

        result = presenter.set_computer_braille_at_cursor_is_enabled(False)
        assert result is True
        assert presenter.get_computer_braille_at_cursor_is_enabled() is False

    def test_get_word_wrap_is_enabled(self, test_context: OrcaTestContext):
        """Test getting word wrap enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_word_wrap_is_enabled() is False

        presenter.set_word_wrap_is_enabled(True)
        assert presenter.get_word_wrap_is_enabled() is True

    def test_set_word_wrap_is_enabled(self, test_context: OrcaTestContext):
        """Test setting word wrap enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_word_wrap_is_enabled(True)
        assert result is True
        assert presenter.get_word_wrap_is_enabled() is True

        result = presenter.set_word_wrap_is_enabled(False)
        assert result is True
        assert presenter.get_word_wrap_is_enabled() is False

    def test_get_flash_messages_are_enabled(self, test_context: OrcaTestContext):
        """Test getting flash messages enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_flash_messages_are_enabled() is True

        presenter.set_flash_messages_are_enabled(False)
        assert presenter.get_flash_messages_are_enabled() is False

    def test_set_flash_messages_are_enabled(self, test_context: OrcaTestContext):
        """Test setting flash messages enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_flash_messages_are_enabled(True)
        assert result is True
        assert presenter.get_flash_messages_are_enabled() is True

        result = presenter.set_flash_messages_are_enabled(False)
        assert result is True
        assert presenter.get_flash_messages_are_enabled() is False

    def test_get_flashtime_from_settings(self, test_context: OrcaTestContext):
        """Test getting flashtime from settings."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        presenter.set_flash_messages_are_persistent(True)
        assert presenter.get_flashtime_from_settings() == -1

        presenter.set_flash_messages_are_persistent(False)
        presenter.set_flash_message_duration(3000)
        assert presenter.get_flashtime_from_settings() == 3000

    def test_get_flash_message_duration(self, test_context: OrcaTestContext):
        """Test getting flash message duration."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_flash_message_duration() == 5000

    def test_set_flash_message_duration(self, test_context: OrcaTestContext):
        """Test setting flash message duration."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_flash_message_duration(4000)
        assert result is True
        assert presenter.get_flash_message_duration() == 4000

    def test_get_flash_messages_are_persistent(self, test_context: OrcaTestContext):
        """Test getting flash messages persistent status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_flash_messages_are_persistent() is False

        presenter.set_flash_messages_are_persistent(True)
        assert presenter.get_flash_messages_are_persistent() is True

    def test_set_flash_messages_are_persistent(self, test_context: OrcaTestContext):
        """Test setting flash messages persistent status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_flash_messages_are_persistent(True)
        assert result is True
        assert presenter.get_flash_messages_are_persistent() is True

        result = presenter.set_flash_messages_are_persistent(False)
        assert result is True
        assert presenter.get_flash_messages_are_persistent() is False

    def test_get_flash_messages_are_detailed(self, test_context: OrcaTestContext):
        """Test getting flash messages detailed status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_flash_messages_are_detailed() is True

        presenter.set_flash_messages_are_detailed(False)
        assert presenter.get_flash_messages_are_detailed() is False

    def test_set_flash_messages_are_detailed(self, test_context: OrcaTestContext):
        """Test setting flash messages detailed status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_flash_messages_are_detailed(True)
        assert result is True
        assert presenter.get_flash_messages_are_detailed() is True

        result = presenter.set_flash_messages_are_detailed(False)
        assert result is True
        assert presenter.get_flash_messages_are_detailed() is False

    def test_get_verbosity_is_detailed(self, test_context: OrcaTestContext):
        """Test _get_verbosity_is_detailed returns correct boolean."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter._get_verbosity_is_detailed() is True

        presenter.set_verbosity_level("brief")
        assert presenter._get_verbosity_is_detailed() is False

    def test_set_verbosity_is_detailed(self, test_context: OrcaTestContext):
        """Test _set_verbosity_is_detailed sets verbosity level from boolean."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter._set_verbosity_is_detailed(True)
        assert result is True
        assert presenter.get_verbosity_level() == "verbose"

        result = presenter._set_verbosity_is_detailed(False)
        assert result is True
        assert presenter.get_verbosity_level() == "brief"

    def test_get_eol_enabled(self, test_context: OrcaTestContext):
        """Test get_end_of_line_indicator_is_enabled returns default then setter value."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_end_of_line_indicator_is_enabled() is True

        presenter.set_end_of_line_indicator_is_enabled(False)
        assert presenter.get_end_of_line_indicator_is_enabled() is False

    def test_set_eol_enabled(self, test_context: OrcaTestContext):
        """Test set_end_of_line_indicator_is_enabled sets end-of-line-indicator."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_end_of_line_indicator_is_enabled(True)
        assert result is True
        assert presenter.get_end_of_line_indicator_is_enabled() is True

        result = presenter.set_end_of_line_indicator_is_enabled(False)
        assert result is True
        assert presenter.get_end_of_line_indicator_is_enabled() is False

    def test_get_use_abbreviated_rolenames(self, test_context: OrcaTestContext):
        """Test _get_use_abbreviated_rolenames returns True when brief style."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter._get_use_abbreviated_rolenames() is False

        presenter.set_rolename_style("brief")
        assert presenter._get_use_abbreviated_rolenames() is True

    def test_set_use_abbreviated_rolenames(self, test_context: OrcaTestContext):
        """Test _set_use_abbreviated_rolenames sets rolename style from boolean."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter._set_use_abbreviated_rolenames(True)
        assert result is True
        assert presenter.get_rolename_style() == "brief"

        result = presenter._set_use_abbreviated_rolenames(False)
        assert result is True
        assert presenter.get_rolename_style() == "verbose"

    def test_get_flash_duration_seconds(self, test_context: OrcaTestContext):
        """Test _get_flash_duration_seconds converts milliseconds to seconds."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter._get_flash_duration_seconds() == 5

        presenter.set_flash_message_duration(2500)
        assert presenter._get_flash_duration_seconds() == 2

        presenter.set_flash_message_duration(1000)
        assert presenter._get_flash_duration_seconds() == 1

    def test_set_flash_duration_seconds(self, test_context: OrcaTestContext):
        """Test _set_flash_duration_seconds converts seconds to milliseconds."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        presenter._set_flash_duration_seconds(5)
        assert presenter.get_flash_message_duration() == 5000

        presenter._set_flash_duration_seconds(1)
        assert presenter.get_flash_message_duration() == 1000

        presenter._set_flash_duration_seconds(10)
        assert presenter.get_flash_message_duration() == 10000

    def test_get_present_mnemonics(self, test_context: OrcaTestContext):
        """Test getting present mnemonics setting."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        assert presenter.get_present_mnemonics() is True

        presenter.set_present_mnemonics(False)
        assert presenter.get_present_mnemonics() is False

    def test_set_present_mnemonics(self, test_context: OrcaTestContext):
        """Test setting present mnemonics."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_present_mnemonics(True)
        assert result is True
        assert presenter.get_present_mnemonics() is True

        result = presenter.set_present_mnemonics(False)
        assert result is True
        assert presenter.get_present_mnemonics() is False

    def test_get_set_monitor_is_enabled(self, test_context: OrcaTestContext):
        """Test getting and setting braille monitor enabled status."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        result = presenter.set_monitor_is_enabled(True)
        assert result is True
        assert presenter.get_monitor_is_enabled() is True

        result = presenter.set_monitor_is_enabled(False)
        assert result is True
        assert presenter.get_monitor_is_enabled() is False

    def test_set_braille_monitor_disabled_destroys_monitor(self, test_context: OrcaTestContext):
        """Test explicitly disabling braille monitor destroys the monitor widget."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        mock_monitor = test_context.Mock()
        presenter._monitor = mock_monitor

        presenter.set_monitor_is_enabled(False)

        mock_monitor.destroy.assert_called_once()
        assert presenter._monitor is None

    def test_init_braille_registers_monitor_callback(self, test_context: OrcaTestContext):
        """Test init_braille registers the monitor callback with braille."""

        essential_modules = self._setup_dependencies(test_context)
        braille_mock = essential_modules["orca.braille"]
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()

        presenter.init_braille()

        braille_mock.set_monitor_callback.assert_called_once_with(presenter.update_monitor)

    def test_update_monitor_creates_when_enabled(self, test_context: OrcaTestContext):
        """Test update_monitor creates monitor on demand when enabled."""

        essential_modules = self._setup_dependencies(test_context)
        braille_monitor_mock = essential_modules["orca.braille_monitor"]
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        presenter.set_monitor_is_enabled(True)
        presenter.set_monitor_cell_count(40)
        mock_monitor = test_context.Mock()
        braille_monitor_mock.BrailleMonitor.return_value = mock_monitor

        presenter.update_monitor(1, "hello", None, 40)

        braille_monitor_mock.BrailleMonitor.assert_called_once_with(
            40,
            on_close=unittest.mock.ANY,
            foreground=unittest.mock.ANY,
            background=unittest.mock.ANY,
        )
        mock_monitor.show_all.assert_called_once()
        mock_monitor.write_text.assert_called_once_with(1, "hello", None)

    def test_update_monitor_uses_cell_count_setting(self, test_context: OrcaTestContext):
        """Test update_monitor uses the configured cell count instead of display size."""

        essential_modules = self._setup_dependencies(test_context)
        braille_monitor_mock = essential_modules["orca.braille_monitor"]
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        presenter.set_monitor_is_enabled(True)
        presenter.set_monitor_cell_count(20)
        mock_monitor = test_context.Mock()
        braille_monitor_mock.BrailleMonitor.return_value = mock_monitor

        presenter.update_monitor(1, "hello", None, 40)

        braille_monitor_mock.BrailleMonitor.assert_called_once_with(
            20,
            on_close=unittest.mock.ANY,
            foreground=unittest.mock.ANY,
            background=unittest.mock.ANY,
        )

    def test_update_monitor_skips_when_disabled(self, test_context: OrcaTestContext):
        """Test update_monitor skips update when disabled without destroying."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        presenter.set_monitor_is_enabled(False)
        mock_monitor = test_context.Mock()
        presenter._monitor = mock_monitor

        presenter.update_monitor(1, "hello", None, 40)

        mock_monitor.write_text.assert_not_called()
        assert presenter._monitor is mock_monitor

    def test_destroy_monitor(self, test_context: OrcaTestContext):
        """Test destroy_monitor destroys existing monitor."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        mock_monitor = test_context.Mock()
        presenter._monitor = mock_monitor

        presenter.destroy_monitor()

        mock_monitor.destroy.assert_called_once()
        assert presenter._monitor is None

    def test_destroy_monitor_no_op_when_none(self, test_context: OrcaTestContext):
        """Test destroy_monitor does nothing when no monitor exists."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        assert presenter._monitor is None

        presenter.destroy_monitor()

        assert presenter._monitor is None

    def test_shutdown_braille_does_not_destroy_monitor(self, test_context: OrcaTestContext):
        """Test shutdown_braille does not destroy the monitor."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        mock_monitor = test_context.Mock()
        presenter._monitor = mock_monitor

        presenter.shutdown_braille()

        mock_monitor.destroy.assert_not_called()
        assert presenter._monitor is mock_monitor
        essential_modules["orca.braille"].shutdown.assert_called_once()

    def test_set_braille_disabled_keeps_monitor(self, test_context: OrcaTestContext):
        """Test disabling braille does not destroy the monitor."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import get_presenter

        presenter = get_presenter()
        mock_monitor = test_context.Mock()
        presenter._monitor = mock_monitor

        presenter.set_braille_is_enabled(False)

        mock_monitor.destroy.assert_not_called()
        assert presenter._monitor is mock_monitor


@pytest.mark.unit
class TestBraillePreferencesGridUI:
    """Test Braille preferences grid UI creation."""

    # pylint: disable-next=too-many-statements
    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Set up mocks for braille_presenter GUI dependencies."""

        additional_modules = ["orca.braille", "orca.braille_monitor", "orca.orca_platform"]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        platform_mock = essential_modules["orca.orca_platform"]
        platform_mock.tablesdir = "/usr/share/liblouis/tables"

        from orca import gsettings_registry

        registry = gsettings_registry.get_registry()
        registry.clear_runtime_values()

        test_context.patch(
            "orca.braille_presenter.BraillePresenter._get_table_files",
            return_value=[
                "en-us-g1.ctb",
                "en-us-g2.ctb",
            ],
        )

        guilabels_mock = essential_modules["orca.guilabels"]
        guilabels_mock.OBJECT_PRESENTATION_IS_DETAILED = "Detailed"
        guilabels_mock.BRAILLE_SHOW_CONTEXT = "Show context"
        guilabels_mock.BRAILLE_ABBREVIATED_ROLE_NAMES = "Abbreviated role names"
        guilabels_mock.PRESENT_OBJECT_MNEMONICS = "Present mnemonics"
        guilabels_mock.VERBOSITY = "Verbosity"
        guilabels_mock.BRAILLE_ENABLE_CONTRACTED_BRAILLE = "Enable contracted braille"
        guilabels_mock.BRAILLE_COMPUTER_BRAILLE_AT_CURSOR = "Expand word at cursor"
        guilabels_mock.BRAILLE_ENABLE_END_OF_LINE_SYMBOL = "End of line symbol"
        guilabels_mock.BRAILLE_ENABLE_WORD_WRAP = "Enable word wrap"
        guilabels_mock.BRAILLE_CONTRACTION_TABLE = "Contraction table"
        guilabels_mock.BRAILLE_HYPERLINK_INDICATOR = "Hyperlink indicator"
        guilabels_mock.BRAILLE_DOT_NONE = "None"
        guilabels_mock.BRAILLE_DOT_7 = "Dot 7"
        guilabels_mock.BRAILLE_DOT_8 = "Dot 8"
        guilabels_mock.BRAILLE_DOT_7_8 = "Dots 7 and 8"
        guilabels_mock.BRAILLE_INDICATORS = "Indicators"
        guilabels_mock.BRAILLE_SELECTION_INDICATOR = "Selection indicator"
        guilabels_mock.BRAILLE_TEXT_ATTRIBUTES_INDICATOR = "Text attributes indicator"
        guilabels_mock.BRAILLE_DISPLAY_SETTINGS = "Display Settings"
        guilabels_mock.BRAILLE_MESSAGES_ARE_PERSISTENT = "Messages are persistent"
        guilabels_mock.BRAILLE_ENABLE_FLASH_MESSAGES = "Enable flash messages"
        guilabels_mock.BRAILLE_MESSAGES_ARE_DETAILED = "Messages are detailed"
        guilabels_mock.BRAILLE_DURATION_SECS = "Duration (secs)"
        guilabels_mock.BRAILLE_FLASH_MESSAGES = "Flash Messages"
        guilabels_mock.GENERAL_BRAILLE_UPDATES = "Braille updates"
        guilabels_mock.GENERAL_FREQUENCY_SECS = "Frequency (secs)"
        guilabels_mock.GENERAL_APPLIES_TO = "Applies to"
        guilabels_mock.PROGRESS_BAR_ALL = "All"
        guilabels_mock.PROGRESS_BAR_APPLICATION = "Application"
        guilabels_mock.PROGRESS_BAR_WINDOW = "Window"
        guilabels_mock.PROGRESS_BARS = "Progress Bars"
        guilabels_mock.BRAILLE = "Braille"
        guilabels_mock.BRAILLE_MONITOR = "On-screen braille"
        guilabels_mock.ON_SCREEN_DISPLAY = "On-Screen Display"
        guilabels_mock.BRAILLE_MONITOR_CELL_COUNT = "Cell count"
        guilabels_mock.BRAILLE_MONITOR_SHOW_DOTS = "Show braille dot patterns"
        guilabels_mock.BRAILLE_MONITOR_FOREGROUND = "Text color"
        guilabels_mock.BRAILLE_MONITOR_BACKGROUND = "Background color"
        guilabels_mock.BRAILLE_MONITOR_INFO = "On-screen braille info"

        return essential_modules

    def test_braille_verbosity_grid_creates_widgets(self, test_context: OrcaTestContext) -> None:
        """Test BrailleVerbosityPreferencesGrid creates correct widgets."""

        from gi.repository import Gtk

        self._setup_dependencies(test_context)
        from orca.braille_presenter import (
            BraillePresenter,
            BrailleVerbosityPreferencesGrid,
        )

        presenter = BraillePresenter()
        grid = BrailleVerbosityPreferencesGrid(presenter)

        assert isinstance(grid, Gtk.Grid)
        assert len(grid._widgets) == 4

    def test_braille_display_settings_grid_creates_widgets(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test BrailleDisplaySettingsPreferencesGrid creates correct widgets."""

        from gi.repository import Gtk

        self._setup_dependencies(test_context)
        from orca.braille_presenter import (
            BrailleDisplaySettingsPreferencesGrid,
            BraillePresenter,
        )

        presenter = BraillePresenter()
        grid = BrailleDisplaySettingsPreferencesGrid(presenter)

        assert isinstance(grid, Gtk.Grid)
        assert len(grid._widgets) == 8

    def test_braille_display_settings_grid_has_contracted_control(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test BrailleDisplaySettingsPreferencesGrid has contracted control."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import (
            BrailleDisplaySettingsPreferencesGrid,
            BraillePresenter,
        )

        presenter = BraillePresenter()
        grid = BrailleDisplaySettingsPreferencesGrid(presenter)

        contracted_switch = grid.get_widget(2)
        assert contracted_switch is not None

        computer_braille_at_cursor_switch = grid.get_widget(3)
        assert computer_braille_at_cursor_switch is not None

        contraction_table_combo = grid.get_widget(4)
        assert contraction_table_combo is not None

    def test_braille_flash_messages_grid_creates_widgets(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test BrailleFlashMessagesPreferencesGrid creates correct widgets."""

        from gi.repository import Gtk

        self._setup_dependencies(test_context)
        from orca.braille_presenter import (
            BrailleFlashMessagesPreferencesGrid,
            BraillePresenter,
        )

        presenter = BraillePresenter()
        grid = BrailleFlashMessagesPreferencesGrid(presenter)

        assert isinstance(grid, Gtk.Grid)
        assert len(grid._widgets) == 4

    def test_braille_flash_messages_grid_has_persistent_control(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test BrailleFlashMessagesPreferencesGrid has persistent control."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import (
            BrailleFlashMessagesPreferencesGrid,
            BraillePresenter,
        )

        presenter = BraillePresenter()
        grid = BrailleFlashMessagesPreferencesGrid(presenter)

        persistent_switch = grid.get_widget(2)
        assert persistent_switch is not None

        duration_spinbutton = grid.get_widget(3)
        assert duration_spinbutton is not None

    def test_braille_progress_bars_grid_creates_widgets(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test BrailleProgressBarsPreferencesGrid creates correct widgets."""

        from gi.repository import Gtk

        self._setup_dependencies(test_context)
        from orca.braille_presenter import (
            BraillePresenter,
            BrailleProgressBarsPreferencesGrid,
        )

        presenter = BraillePresenter()
        grid = BrailleProgressBarsPreferencesGrid(presenter)

        assert isinstance(grid, Gtk.Grid)
        assert len(grid._widgets) == 3

    def test_braille_preferences_grid_creates_multi_page_stack(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test BraillePreferencesGrid creates multi-page stack."""

        from gi.repository import Gtk

        self._setup_dependencies(test_context)
        from orca.braille_presenter import BraillePreferencesGrid, BraillePresenter

        presenter = BraillePresenter()
        grid = BraillePreferencesGrid(presenter)

        assert isinstance(grid, Gtk.Grid)
        assert grid._verbosity_grid is not None
        assert grid._display_settings_grid is not None
        assert grid._flash_messages_grid is not None
        assert grid._progress_bars_grid is not None

    def test_braille_preferences_grid_save_settings(self, test_context: OrcaTestContext) -> None:
        """Test BraillePreferencesGrid save_settings returns combined dict."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import BraillePreferencesGrid, BraillePresenter

        presenter = BraillePresenter()
        grid = BraillePreferencesGrid(presenter)

        result = grid.save_settings()

        assert isinstance(result, dict)
        assert "verbosity-level" in result
        assert "end-of-line-indicator" in result
        assert "flash-messages" in result
        assert "braille-progress-bar-updates" in result

    def test_braille_preferences_grid_title_callback(self, test_context: OrcaTestContext) -> None:
        """Test BraillePreferencesGrid stores title change callback."""

        self._setup_dependencies(test_context)
        from orca.braille_presenter import BraillePreferencesGrid, BraillePresenter

        presenter = BraillePresenter()
        callback_calls: list[str] = []

        def title_callback(title: str) -> None:
            callback_calls.append(title)

        grid = BraillePreferencesGrid(presenter, title_change_callback=title_callback)

        assert grid._title_change_callback is title_callback

    def test_verbosity_grid_switch_reflects_initial_value(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test verbosity grid switch shows correct initial value."""

        self._setup_dependencies(test_context)

        from orca.braille_presenter import (
            BraillePresenter,
            BrailleVerbosityPreferencesGrid,
        )

        presenter = BraillePresenter()
        presenter.set_verbosity_level("verbose")
        grid = BrailleVerbosityPreferencesGrid(presenter)

        detailed_switch = grid.get_widget(0)
        assert detailed_switch is not None
        assert detailed_switch.get_active() is True

    def test_verbosity_grid_switch_toggle_updates_setting(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test toggling verbosity switch updates the setting."""

        self._setup_dependencies(test_context)

        from orca.braille_presenter import (
            BraillePresenter,
            BrailleVerbosityPreferencesGrid,
        )

        presenter = BraillePresenter()
        presenter.set_verbosity_level("brief")
        grid = BrailleVerbosityPreferencesGrid(presenter)

        detailed_switch = grid.get_widget(0)
        assert detailed_switch is not None
        assert detailed_switch.get_active() is False

        grid._initializing = False
        detailed_switch.set_active(True)

        result = grid.save_settings()
        assert result["verbosity-level"] == "verbose"

    def test_flash_messages_duration_initial_value(self, test_context: OrcaTestContext) -> None:
        """Test flash duration spinbutton shows correct initial value."""

        self._setup_dependencies(test_context)

        from orca.braille_presenter import (
            BrailleFlashMessagesPreferencesGrid,
            BraillePresenter,
        )

        presenter = BraillePresenter()
        presenter.set_flash_message_duration(3000)
        grid = BrailleFlashMessagesPreferencesGrid(presenter)

        duration_spinbutton = grid.get_widget(3)
        assert duration_spinbutton is not None
        assert duration_spinbutton.get_value() == 3

    def test_display_settings_combobox_initial_value(self, test_context: OrcaTestContext) -> None:
        """Test indicator combobox shows correct initial selection."""

        self._setup_dependencies(test_context)

        from orca.braille_presenter import (
            BrailleDisplaySettingsPreferencesGrid,
            BraillePresenter,
        )

        presenter = BraillePresenter()
        presenter.set_link_indicator("dot7")
        grid = BrailleDisplaySettingsPreferencesGrid(presenter)

        link_combo = grid.get_widget(5)
        assert link_combo is not None
        assert link_combo.get_active() == 1



# Unit tests for chat_presenter.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=protected-access

"""Unit tests for chat_presenter.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING

import gi
import pytest

gi.require_version("Atspi", "2.0")
from gi.repository import Atspi

if TYPE_CHECKING:
    from unittest.mock import Mock

    from .orca_test_context import OrcaTestContext


@pytest.mark.unit
class TestConversation:
    """Test Conversation class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, Mock]:
        """Returns dependencies for chat module testing."""

        essential_modules = test_context.setup_shared_dependencies(
            [
                "orca.ax_utilities",
                "orca.ax_text",
                "orca.input_event_manager",
                "orca.braille_presenter",
                "orca.presentation_manager",
            ],
        )

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_message = test_context.Mock()
        debug_mock.print_tokens = test_context.Mock()

        return essential_modules

    def test_init(self, test_context: OrcaTestContext) -> None:
        """Test Conversation.__init__."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        assert conversation.get_name() == "TestRoom"
        assert conversation._log == mock_log
        assert not conversation.has_messages()
        assert conversation.get_message_count() == 0

    def test_get_name(self, test_context: OrcaTestContext) -> None:
        """Test Conversation.get_name returns the conversation name."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("MyChannel", mock_log)

        assert conversation.get_name() == "MyChannel"

    def test_is_log(self, test_context: OrcaTestContext) -> None:
        """Test Conversation.is_log identifies the log object."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        other_obj = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        assert conversation.is_log(mock_log) is True
        assert conversation.is_log(other_obj) is False

    def test_add_and_get_message(self, test_context: OrcaTestContext) -> None:
        """Test Conversation.add_message and get_message."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        conversation.add_message("Hello")
        conversation.add_message("World")

        assert conversation.get_message(-1) == "World"
        assert conversation.get_message(-2) == "Hello"

    def test_has_messages_and_count(self, test_context: OrcaTestContext) -> None:
        """Test Conversation.has_messages and get_message_count."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        assert not conversation.has_messages()
        assert conversation.get_message_count() == 0

        conversation.add_message("Message 1")
        assert conversation.has_messages()
        assert conversation.get_message_count() == 1

        conversation.add_message("Message 2")
        assert conversation.get_message_count() == 2

    def test_message_history_limit(self, test_context: OrcaTestContext) -> None:
        """Test that Conversation respects HISTORY_SIZE limit."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        # Add more messages than HISTORY_SIZE
        for i in range(Conversation.HISTORY_SIZE + 3):
            conversation.add_message(f"Message {i}")

        assert conversation.get_message_count() == Conversation.HISTORY_SIZE
        # Oldest messages should have been dropped
        assert conversation.get_message(-1) == f"Message {Conversation.HISTORY_SIZE + 2}"

    def test_typing_status(self, test_context: OrcaTestContext) -> None:
        """Test Conversation typing status methods."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        assert conversation.get_typing_status() == ""

        conversation.set_typing_status("User is typing...")
        assert conversation.get_typing_status() == "User is typing..."

    def test_get_message_invalid_index(self, test_context: OrcaTestContext) -> None:
        """Test Conversation.get_message raises IndexError for invalid index."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        # Empty conversation should raise IndexError
        with pytest.raises(IndexError):
            conversation.get_message(-1)

        # Add one message
        conversation.add_message("Hello")

        # Valid indices work
        assert conversation.get_message(-1) == "Hello"

        # Out of bounds still raises
        with pytest.raises(IndexError):
            conversation.get_message(-2)


@pytest.mark.unit
class TestConversationList:
    """Test ConversationList class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, Mock]:
        """Returns dependencies for chat module testing."""

        essential_modules = test_context.setup_shared_dependencies(
            [
                "orca.ax_utilities",
                "orca.ax_text",
                "orca.input_event_manager",
                "orca.braille_presenter",
                "orca.presentation_manager",
            ],
        )

        return essential_modules

    def test_init(self, test_context: OrcaTestContext) -> None:
        """Test ConversationList.__init__."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import ConversationList

        conv_list = ConversationList()

        assert not conv_list.has_messages()
        assert conv_list.get_message_count() == 0

    def test_add_message_without_conversation(self, test_context: OrcaTestContext) -> None:
        """Test ConversationList.add_message with no conversation."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import ConversationList

        conv_list = ConversationList()
        conv_list.add_message("Hello", None)

        assert conv_list.has_messages()
        assert conv_list.get_message_count() == 1

        message, name, log = conv_list.get_message_and_name(-1)
        assert message == "Hello"
        assert name == ""
        assert log is None

    def test_add_message_with_conversation(self, test_context: OrcaTestContext) -> None:
        """Test ConversationList.add_message with a conversation."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation, ConversationList

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)

        conv_list = ConversationList()
        conv_list.add_message("Hello from room", conversation)

        message, name, log = conv_list.get_message_and_name(-1)
        assert message == "Hello from room"
        assert name == "TestRoom"
        assert log is mock_log

    def test_iteration(self, test_context: OrcaTestContext) -> None:
        """Test ConversationList iteration."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation, ConversationList

        mock_log1 = test_context.Mock(spec=Atspi.Accessible)
        mock_log2 = test_context.Mock(spec=Atspi.Accessible)
        conv1 = Conversation("Room1", mock_log1)
        conv2 = Conversation("Room2", mock_log2)

        conv_list = ConversationList()
        conv_list.add_message("Msg1", conv1)
        conv_list.add_message("Msg2", conv2)

        conversations = list(conv_list)
        assert len(conversations) == 2
        assert conv1 in conversations
        assert conv2 in conversations

    def test_iteration_empty(self, test_context: OrcaTestContext) -> None:
        """Test ConversationList iteration when empty."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import ConversationList

        conv_list = ConversationList()
        conversations = list(conv_list)
        assert len(conversations) == 0

    def test_message_history_limit(self, test_context: OrcaTestContext) -> None:
        """Test that ConversationList respects HISTORY_SIZE limit."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Conversation, ConversationList

        conv_list = ConversationList()

        # Add more messages than HISTORY_SIZE
        for i in range(Conversation.HISTORY_SIZE + 3):
            conv_list.add_message(f"Message {i}", None)

        assert conv_list.get_message_count() == Conversation.HISTORY_SIZE

    def test_get_message_and_name_invalid_index(self, test_context: OrcaTestContext) -> None:
        """Test ConversationList.get_message_and_name raises IndexError for invalid index."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import ConversationList

        conv_list = ConversationList()

        # Empty list should raise IndexError
        with pytest.raises(IndexError):
            conv_list.get_message_and_name(-1)

        # Add one message
        conv_list.add_message("Hello", None)

        # Valid index works
        message, _, _ = conv_list.get_message_and_name(-1)
        assert message == "Hello"

        # Out of bounds still raises
        with pytest.raises(IndexError):
            conv_list.get_message_and_name(-2)


@pytest.mark.unit
class TestChat:
    """Test Chat class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, Mock]:
        """Returns dependencies for Chat class testing."""

        essential_modules = test_context.setup_shared_dependencies(
            [
                "orca.ax_utilities",
                "orca.ax_text",
                "orca.input_event_manager",
                "orca.braille_presenter",
                "orca.presentation_manager",
            ],
        )

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_message = test_context.Mock()
        debug_mock.print_tokens = test_context.Mock()

        cmdnames_mock = essential_modules["orca.cmdnames"]
        cmdnames_mock.CHAT_TOGGLE_ROOM_NAME_PREFIX = "Toggle room name prefix"
        cmdnames_mock.CHAT_TOGGLE_BUDDY_TYPING = "Toggle buddy typing"
        cmdnames_mock.CHAT_TOGGLE_MESSAGE_HISTORIES = "Toggle message histories"
        cmdnames_mock.CHAT_PREVIOUS_MESSAGE = "Previous message"
        cmdnames_mock.CHAT_NEXT_MESSAGE = "Next message"

        messages_mock = essential_modules["orca.messages"]
        messages_mock.CHAT_NO_MESSAGES = "No chat messages"
        messages_mock.CHAT_LIST_TOP = "Top"
        messages_mock.CHAT_LIST_BOTTOM = "Bottom"
        messages_mock.CHAT_ROOM_NAME_PREFIX_ON = "Room name prefix on"
        messages_mock.CHAT_ROOM_NAME_PREFIX_OFF = "Room name prefix off"
        messages_mock.CHAT_BUDDY_TYPING_ON = "Buddy typing on"
        messages_mock.CHAT_BUDDY_TYPING_OFF = "Buddy typing off"
        messages_mock.CHAT_SEPARATE_HISTORIES_ON = "Separate histories on"
        messages_mock.CHAT_SEPARATE_HISTORIES_OFF = "Separate histories off"

        input_event_mock = essential_modules["orca.input_event"]
        input_event_handler_mock = test_context.Mock()
        input_event_mock.InputEventHandler = test_context.Mock(
            return_value=input_event_handler_mock,
        )

        keybindings_mock = essential_modules["orca.keybindings"]
        keybindings_mock.NO_MODIFIER_MASK = 0
        keybindings_mock.ORCA_MODIFIER_MASK = 4

        from orca import gsettings_registry

        registry = gsettings_registry.get_registry()
        registry.clear_runtime_values()

        return essential_modules

    def test_init(self, test_context: OrcaTestContext) -> None:
        """Test Chat.__init__."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Chat

        mock_script = test_context.Mock()
        chat = Chat(mock_script)

        assert chat._script == mock_script

    def test_get_conversation_for_object_name_mismatch(self, test_context: OrcaTestContext) -> None:
        """Test get_conversation_for_object when name doesn't match log's room."""

        self._setup_dependencies(test_context)
        from orca.chat_presenter import Chat, Conversation

        mock_script = test_context.Mock()
        chat = Chat(mock_script)

        mock_log1 = test_context.Mock(spec=Atspi.Accessible)
        conversation1 = Conversation("Room1", mock_log1)
        chat._conversation_list.add_message("Hello", conversation1)

        test_context.patch_object(chat, "get_chat_room_name", return_value="Room2")

        result = chat.get_conversation_for_object(mock_log1)

        assert result is None


@pytest.mark.unit
class TestChatPresenter:
    """Test ChatPresenter class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, Mock]:
        """Returns dependencies for ChatPresenter class testing."""

        essential_modules = test_context.setup_shared_dependencies(
            [
                "orca.ax_utilities",
                "orca.ax_text",
                "orca.input_event_manager",
                "orca.braille_presenter",
                "orca.presentation_manager",
            ],
        )

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_message = test_context.Mock()
        debug_mock.print_tokens = test_context.Mock()

        cmdnames_mock = essential_modules["orca.cmdnames"]
        cmdnames_mock.CHAT_TOGGLE_ROOM_NAME_PREFIX = "Toggle room name prefix"
        cmdnames_mock.CHAT_TOGGLE_BUDDY_TYPING = "Toggle buddy typing"
        cmdnames_mock.CHAT_TOGGLE_MESSAGE_HISTORIES = "Toggle message histories"
        cmdnames_mock.CHAT_PREVIOUS_MESSAGE = "Previous message"
        cmdnames_mock.CHAT_NEXT_MESSAGE = "Next message"

        messages_mock = essential_modules["orca.messages"]
        messages_mock.CHAT_ROOM_NAME_PREFIX_ON = "Room name prefix on"
        messages_mock.CHAT_ROOM_NAME_PREFIX_OFF = "Room name prefix off"
        messages_mock.CHAT_BUDDY_TYPING_ON = "Buddy typing on"
        messages_mock.CHAT_BUDDY_TYPING_OFF = "Buddy typing off"
        messages_mock.CHAT_SEPARATE_HISTORIES_ON = "Separate histories on"
        messages_mock.CHAT_SEPARATE_HISTORIES_OFF = "Separate histories off"
        messages_mock.CHAT_NO_MESSAGES = "No messages"
        messages_mock.CHAT_LIST_TOP = "Top of chat"
        messages_mock.CHAT_LIST_BOTTOM = "Bottom of chat"
        messages_mock.CHAT_MESSAGE_FROM_ROOM = "Message from %s"

        input_event_mock = essential_modules["orca.input_event"]
        input_event_handler_mock = test_context.Mock()
        input_event_mock.InputEventHandler = test_context.Mock(
            return_value=input_event_handler_mock,
        )

        keybindings_mock = essential_modules["orca.keybindings"]
        keybindings_mock.NO_MODIFIER_MASK = 0
        keybindings_mock.ORCA_MODIFIER_MASK = 4
        keybindings_mock.DEFAULT_MODIFIER_MASK = 8

        from orca import gsettings_registry

        registry = gsettings_registry.get_registry()
        registry.clear_runtime_values()

        return essential_modules

    def test_get_handlers(self, test_context: OrcaTestContext) -> None:
        """Test ChatPresenter registers commands with CommandManager."""

        self._setup_dependencies(test_context)
        from orca import command_manager
        from orca.chat_presenter import get_presenter

        presenter = get_presenter()
        presenter.set_up_commands()
        cmd_manager = command_manager.get_manager()

        assert cmd_manager.get_keyboard_command("chat_previous_message") is not None
        assert cmd_manager.get_keyboard_command("chat_next_message") is not None
        assert cmd_manager.get_keyboard_command("chat_toggle_room_name_prefix") is not None
        assert cmd_manager.get_keyboard_command("chat_toggle_buddy_typing") is not None
        assert cmd_manager.get_keyboard_command("chat_toggle_message_histories") is not None

    def test_toggle_prefix_on_to_off(self, test_context: OrcaTestContext) -> None:
        """Test toggle_prefix from on to off."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import get_presenter

        mock_script = test_context.Mock()

        presenter = get_presenter()
        presenter.set_speak_room_name(True)
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.toggle_prefix(mock_script, None)

        assert result is True
        assert presenter.get_speak_room_name() is False
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_ROOM_NAME_PREFIX_OFF,
        )

    def test_toggle_prefix_off_to_on(self, test_context: OrcaTestContext) -> None:
        """Test toggle_prefix from off to on."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import get_presenter

        mock_script = test_context.Mock()

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.toggle_prefix(mock_script, None)

        assert result is True
        assert presenter.get_speak_room_name() is True
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_ROOM_NAME_PREFIX_ON,
        )

    def test_toggle_buddy_typing_on_to_off(self, test_context: OrcaTestContext) -> None:
        """Test toggle_buddy_typing from on to off."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import get_presenter

        mock_script = test_context.Mock()

        presenter = get_presenter()
        presenter.set_announce_buddy_typing(True)
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.toggle_buddy_typing(mock_script, None)

        assert result is True
        assert presenter.get_announce_buddy_typing() is False
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_BUDDY_TYPING_OFF,
        )

    def test_toggle_buddy_typing_off_to_on(self, test_context: OrcaTestContext) -> None:
        """Test toggle_buddy_typing from off to on."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import get_presenter

        mock_script = test_context.Mock()

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.toggle_buddy_typing(mock_script, None)

        assert result is True
        assert presenter.get_announce_buddy_typing() is True
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_BUDDY_TYPING_ON,
        )

    def test_toggle_message_histories_on_to_off(self, test_context: OrcaTestContext) -> None:
        """Test toggle_message_histories from on to off."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import get_presenter

        mock_script = test_context.Mock()

        presenter = get_presenter()
        presenter.set_room_histories(True)
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.toggle_message_histories(mock_script, None)

        assert result is True
        assert presenter.get_room_histories() is False
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_SEPARATE_HISTORIES_OFF,
        )

    def test_toggle_message_histories_off_to_on(self, test_context: OrcaTestContext) -> None:
        """Test toggle_message_histories from off to on."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import get_presenter

        mock_script = test_context.Mock()

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.toggle_message_histories(mock_script, None)

        assert result is True
        assert presenter.get_room_histories() is True
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_SEPARATE_HISTORIES_ON,
        )

    def _setup_navigation_mocks(
        self,
        test_context: OrcaTestContext,
        essential_modules: dict[str, Mock],
    ) -> Mock:
        """Setup additional mocks needed for navigation tests. Returns mock_script."""

        script_manager_mock = essential_modules["orca.script_manager"]
        script_manager_instance = script_manager_mock.get_manager.return_value
        script_manager_instance.get_active_script = test_context.Mock(return_value=None)

        mock_script = test_context.Mock()
        speech_gen = test_context.Mock()
        speech_gen.voice = test_context.Mock(return_value=None)
        mock_script.get_speech_generator = test_context.Mock(return_value=speech_gen)
        mock_script.speak_message = test_context.Mock()
        mock_script.display_message = test_context.Mock()
        mock_script.app = test_context.Mock()

        return mock_script

    def test_present_previous_no_messages(self, test_context: OrcaTestContext) -> None:
        """Test present_previous_message with no messages."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import Chat, get_presenter

        mock_script = test_context.Mock()
        chat = Chat(mock_script)
        mock_script.chat = chat

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.present_previous_message(mock_script, None)

        assert result is True
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_NO_MESSAGES,
        )

    def test_present_next_no_messages(self, test_context: OrcaTestContext) -> None:
        """Test present_next_message with no messages."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import Chat, get_presenter

        mock_script = test_context.Mock()
        chat = Chat(mock_script)
        mock_script.chat = chat

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()
        result = presenter.present_next_message(mock_script, None)

        assert result is True
        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_NO_MESSAGES,
        )

    def test_navigation_with_messages(self, test_context: OrcaTestContext) -> None:
        """Test navigation through message history."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import Chat, Conversation, get_presenter

        mock_script = self._setup_navigation_mocks(test_context, essential_modules)
        chat = Chat(mock_script)
        mock_script.chat = chat

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)
        chat._conversation_list.add_message("Message 1", conversation)
        chat._conversation_list.add_message("Message 2", conversation)
        chat._conversation_list.add_message("Message 3", conversation)

        presenter = get_presenter()
        presenter.present_previous_message(mock_script, None)
        assert chat._current_index == -1

        presenter.present_previous_message(mock_script, None)
        assert chat._current_index == -2

        presenter.present_next_message(mock_script, None)
        assert chat._current_index == -1

    def test_navigation_top_boundary(self, test_context: OrcaTestContext) -> None:
        """Test navigation stops at top boundary."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import Chat, Conversation, get_presenter

        mock_script = self._setup_navigation_mocks(test_context, essential_modules)
        chat = Chat(mock_script)
        mock_script.chat = chat

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)
        chat._conversation_list.add_message("Message 1", conversation)
        chat._conversation_list.add_message("Message 2", conversation)

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        presenter.present_previous_message(mock_script, None)
        presenter.present_previous_message(mock_script, None)

        pres_manager.present_message.reset_mock()
        presenter.present_previous_message(mock_script, None)

        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_LIST_TOP,
        )

    def test_navigation_bottom_boundary(self, test_context: OrcaTestContext) -> None:
        """Test navigation stops at bottom boundary."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.chat_presenter import Chat, Conversation, get_presenter

        mock_script = self._setup_navigation_mocks(test_context, essential_modules)
        chat = Chat(mock_script)
        mock_script.chat = chat

        mock_log = test_context.Mock(spec=Atspi.Accessible)
        conversation = Conversation("TestRoom", mock_log)
        chat._conversation_list.add_message("Message 1", conversation)
        chat._conversation_list.add_message("Message 2", conversation)

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        presenter.present_previous_message(mock_script, None)

        pres_manager.present_message.reset_mock()
        presenter.present_next_message(mock_script, None)

        pres_manager.present_message.assert_called_with(
            essential_modules["orca.messages"].CHAT_LIST_BOTTOM,
        )

    def _setup_verbosity_test(
        self,
        test_context: OrcaTestContext,
        verbosity: int,
    ) -> tuple[dict[str, Mock], Mock]:
        """Sets up mocks for utter_message verbosity tests. Returns (modules, script)."""

        essential_modules = self._setup_dependencies(test_context)
        mock_script = self._setup_navigation_mocks(test_context, essential_modules)

        nick_map = {0: "all", 1: "all-if-focused", 2: "focused-channel", 3: "active-channel"}
        from orca import gsettings_registry

        gsettings_registry.get_registry().set_runtime_value(
            "chat",
            "message-verbosity",
            nick_map[verbosity],
        )
        return essential_modules, mock_script

    def test_utter_message_all_speaks_regardless(self, test_context: OrcaTestContext) -> None:
        """Test that ALL verbosity speaks regardless of focus or active channel."""

        essential_modules, mock_script = self._setup_verbosity_test(test_context, 0)
        from orca.chat_presenter import get_presenter

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.speak_accessible_text.reset_mock()
        mock_obj = test_context.Mock()
        presenter.utter_message(
            mock_script,
            mock_obj,
            "Room",
            "Hello",
            focused=False,
            active_channel=False,
        )

        pres_manager.speak_accessible_text.assert_called_once()

    def test_utter_message_active_channel_speaks_when_active(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test that ACTIVE_CHANNEL verbosity speaks when active_channel=True."""

        essential_modules, mock_script = self._setup_verbosity_test(test_context, 3)
        from orca.chat_presenter import get_presenter

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.speak_accessible_text.reset_mock()
        mock_obj = test_context.Mock()
        presenter.utter_message(
            mock_script,
            mock_obj,
            "Room",
            "Hello",
            focused=False,
            active_channel=True,
        )

        pres_manager.speak_accessible_text.assert_called_once()

    def test_utter_message_active_channel_silent_when_inactive(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test that ACTIVE_CHANNEL verbosity is silent when active_channel=False."""

        essential_modules, mock_script = self._setup_verbosity_test(test_context, 3)
        from orca.chat_presenter import get_presenter

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.speak_accessible_text.reset_mock()
        mock_obj = test_context.Mock()
        presenter.utter_message(
            mock_script,
            mock_obj,
            "Room",
            "Hello",
            focused=False,
            active_channel=False,
        )

        pres_manager.speak_accessible_text.assert_not_called()

    def test_utter_message_focused_channel_silent_when_unfocused(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test that FOCUSED_CHANNEL is silent when unfocused even if active_channel=True."""

        essential_modules, mock_script = self._setup_verbosity_test(test_context, 2)
        from orca.chat_presenter import get_presenter

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.speak_accessible_text.reset_mock()
        mock_obj = test_context.Mock()
        presenter.utter_message(
            mock_script,
            mock_obj,
            "Room",
            "Hello",
            focused=False,
            active_channel=True,
        )

        pres_manager.speak_accessible_text.assert_not_called()

    def test_utter_message_focused_channel_speaks_when_focused(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test that FOCUSED_CHANNEL speaks when focused=True."""

        essential_modules, mock_script = self._setup_verbosity_test(test_context, 2)
        from orca.chat_presenter import get_presenter

        presenter = get_presenter()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.speak_accessible_text.reset_mock()
        mock_obj = test_context.Mock()
        presenter.utter_message(
            mock_script,
            mock_obj,
            "",
            "Hello",
            focused=True,
            active_channel=True,
        )

        pres_manager.speak_accessible_text.assert_called_once()



# Unit tests for profile_manager.py methods.
#
# Copyright 2025-2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=protected-access

"""Unit tests for profile_manager.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING

import gi

gi.require_version("Gtk", "3.0")

import pytest

if TYPE_CHECKING:
    from unittest.mock import MagicMock

    from .orca_test_context import OrcaTestContext


@pytest.mark.unit
class TestProfileManager:
    """Test ProfileManager methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Set up mocks for profile_manager dependencies."""

        additional_modules = [
            "orca.braille",
            "orca.orca",
            "orca.orca_modifier_manager",
            "orca.speech_manager",
            "orca.braille_presenter",
            "orca.presentation_manager",
        ]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        from orca import gsettings_registry

        registry = gsettings_registry.get_registry()
        registry.set_active_profile("default")

        test_context.patch(
            "orca.profile_manager.ProfileManager._get_stored_profiles",
            return_value=[
                ["Default", "default"],
                ["Spanish", "spanish"],
                ["Work", "work"],
            ],
        )

        speech_manager_mock = essential_modules["orca.speech_manager"]
        speech_manager_mock.get_manager.return_value.refresh_speech.return_value = None

        braille_mock = essential_modules["orca.braille"]
        braille_mock.check_braille_setting.return_value = None

        orca_mock = essential_modules["orca.orca"]
        orca_mock.load_user_settings.return_value = None

        messages_mock = essential_modules["orca.messages"]
        messages_mock.PROFILE_NOT_FOUND = "No profiles found."
        messages_mock.PROFILE_CHANGED = "Profile set to %s."
        messages_mock.PROFILE_CURRENT = "Current profile is %s."

        guilabels_mock = essential_modules["orca.guilabels"]
        guilabels_mock.PROFILE_DEFAULT = "Default"

        return essential_modules

    def test_get_active_profile(self, test_context: OrcaTestContext) -> None:
        """Test getting active profile."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager

        manager = ProfileManager()
        profile = manager.get_active_profile()

        assert profile == "default"

    def test_set_active_profile(self, test_context: OrcaTestContext) -> None:
        """Test setting active profile."""

        self._setup_dependencies(test_context)
        from orca import gsettings_registry
        from orca.profile_manager import ProfileManager

        manager = ProfileManager()
        manager.set_active_profile("spanish")

        assert gsettings_registry.get_registry().get_active_profile() == "spanish"

    def test_load_profile(self, test_context: OrcaTestContext) -> None:
        """Test loading a profile calls set_active_profile and load_user_settings."""

        essential_modules = self._setup_dependencies(test_context)
        from orca import gsettings_registry
        from orca.profile_manager import ProfileManager

        manager = ProfileManager()
        manager.load_profile("spanish")

        assert gsettings_registry.get_registry().get_active_profile() == "spanish"
        essential_modules["orca.orca"].load_user_settings.assert_called_once_with(
            skip_reload_message=True,
        )

    def test_remove_profile(self, test_context: OrcaTestContext) -> None:
        """Test removing a profile."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager

        mock_run = test_context.patch("subprocess.run")
        manager = ProfileManager()
        manager.remove_profile("spanish")

        mock_run.assert_called_once_with(
            ["dconf", "reset", "-f", "/org/gnome/orca/spanish/"],
            check=True,
        )

    def test_remove_profile_dconf_failure(self, test_context: OrcaTestContext) -> None:
        """Test removing a profile when dconf reset fails."""

        import subprocess

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager

        mock_run = test_context.patch("subprocess.run")
        mock_run.side_effect = subprocess.CalledProcessError(1, "dconf")
        manager = ProfileManager()
        manager.remove_profile("spanish")

        mock_run.assert_called_once()

    def test_remove_profile_dconf_not_found(self, test_context: OrcaTestContext) -> None:
        """Test removing a profile when dconf is not installed."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager

        mock_run = test_context.patch("subprocess.run")
        mock_run.side_effect = FileNotFoundError("dconf not found")
        manager = ProfileManager()
        manager.remove_profile("spanish")

        mock_run.assert_called_once()

    def test_remove_profile_sanitizes_name(self, test_context: OrcaTestContext) -> None:
        """Test removing a profile sanitizes the name for the dconf path."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager

        mock_run = test_context.patch("subprocess.run")
        manager = ProfileManager()
        manager.remove_profile("My Profile")

        mock_run.assert_called_once_with(
            ["dconf", "reset", "-f", "/org/gnome/orca/my-profile/"],
            check=True,
        )

    def test_rename_profile(self, test_context: OrcaTestContext) -> None:
        """Test renaming a profile."""

        self._setup_dependencies(test_context)
        from orca import gsettings_registry
        from orca.profile_manager import ProfileManager

        registry = gsettings_registry.get_registry()
        mock_rename = test_context.patch_object(registry, "rename_profile")

        manager = ProfileManager()
        manager.rename_profile("spanish", ["Espanol", "espanol"])

        mock_rename.assert_called_once_with("spanish", "Espanol", "espanol")

    def test_commands_registered(self, test_context: OrcaTestContext) -> None:
        """Test that profile manager commands are registered with CommandManager."""

        self._setup_dependencies(test_context)
        from orca import command_manager
        from orca.profile_manager import ProfileManager

        manager = ProfileManager()
        manager.set_up_commands()
        cmd_manager = command_manager.get_manager()

        assert cmd_manager.get_keyboard_command("cycleSettingsProfileHandler") is not None
        assert cmd_manager.get_keyboard_command("presentCurrentProfileHandler") is not None

    def test_cycle_settings_profile_cycles_to_next(self, test_context: OrcaTestContext) -> None:
        """Test cycle_settings_profile cycles to next profile."""

        essential_modules = self._setup_dependencies(test_context)
        from unittest.mock import MagicMock

        from orca import gsettings_registry
        from orca.profile_manager import ProfileManager

        manager = ProfileManager()
        mock_script = MagicMock()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()

        result = manager.cycle_settings_profile(script=mock_script)

        assert result is True
        assert gsettings_registry.get_registry().get_active_profile() == "spanish"
        pres_manager.present_message.assert_called()

    def test_cycle_settings_profile_wraps_around(self, test_context: OrcaTestContext) -> None:
        """Test cycle_settings_profile wraps to first profile at end."""

        self._setup_dependencies(test_context)
        from unittest.mock import MagicMock

        from orca import gsettings_registry
        from orca.profile_manager import ProfileManager

        gsettings_registry.get_registry().set_active_profile("work")

        manager = ProfileManager()
        mock_script = MagicMock()

        result = manager.cycle_settings_profile(script=mock_script)

        assert result is True
        assert gsettings_registry.get_registry().get_active_profile() == "default"

    def test_cycle_settings_profile_no_profiles(self, test_context: OrcaTestContext) -> None:
        """Test cycle_settings_profile handles no profiles."""

        essential_modules = self._setup_dependencies(test_context)
        from unittest.mock import MagicMock

        from orca.profile_manager import ProfileManager

        manager = ProfileManager()
        mock_script = MagicMock()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()

        result = manager.cycle_settings_profile(script=mock_script)

        assert result is True
        pres_manager.present_message.assert_called()

    def test_present_current_profile(self, test_context: OrcaTestContext) -> None:
        """Test present_current_profile presents the current profile name."""

        essential_modules = self._setup_dependencies(test_context)
        from unittest.mock import MagicMock

        from orca.profile_manager import ProfileManager

        manager = ProfileManager()
        mock_script = MagicMock()
        pres_manager = essential_modules["orca.presentation_manager"].get_manager()
        pres_manager.present_message.reset_mock()

        result = manager.present_current_profile(script=mock_script)

        assert result is True
        pres_manager.present_message.assert_called()
        call_args = pres_manager.present_message.call_args
        assert "Default" in call_args[0][0]


@pytest.mark.unit
class TestProfilePreferencesGridUI:
    """Test ProfilePreferencesGrid UI creation."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Set up mocks for ProfilePreferencesGrid dependencies."""

        additional_modules = [
            "orca.braille",
            "orca.orca",
            "orca.speech_manager",
            "orca.braille_presenter",
            "orca.presentation_manager",
        ]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        from orca import gsettings_registry

        registry = gsettings_registry.get_registry()
        registry.set_active_profile("default")

        test_context.patch(
            "orca.profile_manager.ProfileManager._get_stored_profiles",
            return_value=[
                ["Default", "default"],
                ["Spanish", "spanish"],
            ],
        )

        guilabels_mock = essential_modules["orca.guilabels"]
        guilabels_mock.GENERAL_PROFILES = "Profiles"
        guilabels_mock.GENERAL_START_UP_PROFILE = "Start-up profile"
        guilabels_mock.PROFILE_DEFAULT = "Default"
        guilabels_mock.PROFILE_CONFLICT_MESSAGE = "Profile %s already exists"
        guilabels_mock.PROFILE_REMOVE_LABEL = "Remove Profile"
        guilabels_mock.PROFILE_REMOVE_MESSAGE = "Remove profile %s?"
        guilabels_mock.MENU_REMOVE_PROFILE = "Remove"
        guilabels_mock.MENU_RENAME = "Rename"
        guilabels_mock.PROFILE_SAVE_AS_TITLE = "Save Profile As"
        guilabels_mock.PROFILE_NAME_LABEL = "Profile name:"
        guilabels_mock.DIALOG_CANCEL = "Cancel"
        guilabels_mock.DIALOG_APPLY = "Apply"
        guilabels_mock.DIALOG_ADD = "Add"
        guilabels_mock.PROFILES_INFO = "Select a profile to edit or create a new one."
        guilabels_mock.CURRENT_PROFILE = "Current Profile"
        guilabels_mock.PROFILE_CREATE_NEW = "_Create New Profile"

        return essential_modules

    def test_grid_creates_successfully(self, test_context: OrcaTestContext) -> None:
        """Test ProfilePreferencesGrid creates without error."""

        from gi.repository import Gtk  # pylint: disable=no-name-in-module

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        assert isinstance(grid, Gtk.Grid)

    def test_grid_has_auto_grid(self, test_context: OrcaTestContext) -> None:
        """Test ProfilePreferencesGrid has auto_grid with controls."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        assert grid._auto_grid is not None

    def test_grid_save_settings_returns_dict(self, test_context: OrcaTestContext) -> None:
        """Test save_settings returns a dictionary."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        result = grid.save_settings()

        assert isinstance(result, dict)

    def test_grid_has_changes_initially_false(self, test_context: OrcaTestContext) -> None:
        """Test has_changes returns False initially."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        assert grid.has_changes() is False

    def test_grid_reload_clears_pending_renames(self, test_context: OrcaTestContext) -> None:
        """Test reload clears pending renames."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        grid._pending_renames["old"] = ["New", "new"]
        assert len(grid._pending_renames) == 1

        grid.reload()

        assert len(grid._pending_renames) == 0

    def test_grid_app_specific_disables_startup_setter(self, test_context: OrcaTestContext) -> None:
        """Test app-specific grid disables startup profile setter."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback, is_app_specific=True)

        assert grid._is_app_specific is True

    def test_validate_profile_name_detects_conflict(self, test_context: OrcaTestContext) -> None:
        """Test _validate_profile_name detects existing profile names."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        is_valid, error_msg = grid._validate_profile_name("Default")

        assert is_valid is False
        assert "Default" in error_msg

    def test_validate_profile_name_allows_unique(self, test_context: OrcaTestContext) -> None:
        """Test _validate_profile_name allows unique names."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        is_valid, error_msg = grid._validate_profile_name("NewProfile")

        assert is_valid is True
        assert error_msg == ""

    def test_get_available_profiles_includes_pending_renames(
        self,
        test_context: OrcaTestContext,
    ) -> None:
        """Test _get_available_profiles includes pending renames."""

        self._setup_dependencies(test_context)
        from orca.profile_manager import ProfileManager, ProfilePreferencesGrid

        manager = ProfileManager()

        def callback(_profile):
            return None

        grid = ProfilePreferencesGrid(manager, callback)

        grid._pending_renames["spanish"] = ["Espanol", "spanish"]
        profiles = grid._get_available_profiles()

        profile_names = [p[0] for p in profiles]
        assert "Espanol" in profile_names
        assert "Spanish" not in profile_names



# Orca
#
# Copyright 2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""GSettings fixtures for integration tests."""

from __future__ import annotations

import os
import shutil
import subprocess
import sys
import tempfile
import types
from typing import TYPE_CHECKING

import pytest

if TYPE_CHECKING:
    from collections.abc import Callable, Generator


def _inject_generated_module_stubs() -> None:
    """Injects minimal stubs for meson-generated modules into sys.modules."""

    i18n = types.ModuleType("orca.orca_i18n")
    i18n._ = lambda x: x  # type: ignore[attr-defined]
    i18n.C_ = lambda c, x: x  # type: ignore[attr-defined]
    i18n.ngettext = lambda s, p, n: s if n == 1 else p  # type: ignore[attr-defined]
    sys.modules["orca.orca_i18n"] = i18n

    platform = types.ModuleType("orca.orca_platform")
    platform.version = "0.0.0-test"  # type: ignore[attr-defined]
    platform.revision = ""  # type: ignore[attr-defined]
    platform.prefix = "/usr"  # type: ignore[attr-defined]
    platform.package = "orca"  # type: ignore[attr-defined]
    platform.datadir = "/usr/share"  # type: ignore[attr-defined]
    platform.tablesdir = ""  # type: ignore[attr-defined]
    sys.modules["orca.orca_platform"] = platform


def _build_schemas(source_root: str, output_dir: str) -> None:
    """Generates and compiles GSettings schemas into output_dir."""

    generator = os.path.join(source_root, "tools", "generate_gsettings_schemas.py")
    src_dir = os.path.join(source_root, "src")
    schema_xml = os.path.join(output_dir, "org.gnome.Orca.gschema.xml")

    subprocess.check_call([sys.executable, generator, src_dir, schema_xml])
    subprocess.check_call(["glib-compile-schemas", output_dir])


@pytest.fixture(scope="session", name="gsettings_registry")
def _gsettings_registry() -> Generator:
    """Session-scoped fixture providing a GSettingsRegistry backed by in-memory storage."""

    source_root = os.path.normpath(os.path.join(os.path.dirname(__file__), "..", ".."))
    schema_dir = tempfile.mkdtemp(prefix="orca-test-schemas-")

    try:
        _build_schemas(source_root, schema_dir)
    except (subprocess.CalledProcessError, FileNotFoundError) as error:
        shutil.rmtree(schema_dir, ignore_errors=True)
        pytest.skip(f"Schema generation failed: {error!s}")

    os.environ["GSETTINGS_BACKEND"] = "memory"
    os.environ["GSETTINGS_SCHEMA_DIR"] = schema_dir

    _inject_generated_module_stubs()

    from orca import (
        speech_presenter,  # noqa: F401  # pylint: disable=unused-import,import-outside-toplevel
    )
    from orca.gsettings_registry import get_registry  # pylint: disable=import-outside-toplevel

    yield get_registry()

    shutil.rmtree(schema_dir, ignore_errors=True)
    os.environ.pop("GSETTINGS_SCHEMA_DIR", None)


@pytest.fixture(name="gsettings_handle")
def _gsettings_handle(
    gsettings_registry,
) -> Callable:
    """Factory fixture returning a GSettingsSchemaHandle that resets user keys on teardown."""

    handles_used: list = []

    def _get_handle(schema_name: str):
        handle = gsettings_registry._get_handle(schema_name)
        if handle is None:
            pytest.fail(f"No handle found for schema {schema_name!r}")
        handles_used.append(handle)
        return handle

    yield _get_handle

    for handle in handles_used:
        for gs in handle._cache.values():
            for key in gs.props.settings_schema.list_keys():
                gs.reset(key)


@pytest.fixture(name="gsettings_profile")
def _gsettings_profile(gsettings_registry) -> Generator:
    """Ensures each test starts with the default profile and no active app."""

    gsettings_registry.set_active_profile("default")
    gsettings_registry.set_active_app(None)
    yield gsettings_registry
    gsettings_registry.set_active_profile("default")
    gsettings_registry.set_active_app(None)



# Unit tests for ax_selection.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=too-many-public-methods
# pylint: disable=wrong-import-position
# pylint: disable=protected-access
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=import-outside-toplevel

"""Unit tests for ax_selection.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING

import gi
import pytest

gi.require_version("Atspi", "2.0")
from gi.repository import Atspi, GLib

if TYPE_CHECKING:
    from unittest.mock import MagicMock

    from .orca_test_context import OrcaTestContext


@pytest.mark.unit
class TestAXSelection:
    """Test AXSelection class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Set up mocks for ax_selection dependencies."""

        essential_modules = test_context.setup_shared_dependencies()

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_message = test_context.Mock()
        debug_mock.print_tokens = test_context.Mock()
        debug_mock.LEVEL_INFO = 800

        ax_object_mock = essential_modules["orca.ax_object"]
        ax_object_mock.AXObject = test_context.Mock()
        ax_object_mock.AXObject.supports_selection = test_context.Mock(return_value=True)

        return essential_modules

    @pytest.mark.parametrize(
        "has_selection_support,side_effect,expected_result,expects_debug_call",
        [
            pytest.param(True, lambda obj: 3, 3, True, id="with_support"),
            pytest.param(False, None, 0, False, id="without_support"),
            pytest.param(
                True,
                lambda obj: (_ for _ in ()).throw(GLib.GError("Test error")),
                0,
                True,
                id="glib_error",
            ),
        ],
    )
    def test_get_selected_child_count_scenarios(
        self,
        test_context: OrcaTestContext,
        has_selection_support: bool,
        side_effect,
        expected_result: int,
        expects_debug_call: bool,
    ) -> None:
        """Test AXSelection.get_selected_child_count with various scenarios."""

        mock_accessible = test_context.Mock(spec=Atspi.Accessible)
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca import debug
        from orca.ax_object import AXObject
        from orca.ax_selection import AXSelection

        test_context.patch_object(
            AXObject,
            "supports_selection",
            return_value=has_selection_support,
        )
        if side_effect:
            test_context.patch_object(
                Atspi.Selection,
                "get_n_selected_children",
                side_effect=side_effect,
            )
        test_context.patch_object(
            debug,
            "print_tokens",
            new=essential_modules["orca.debug"].print_tokens,
        )
        result = AXSelection.get_selected_child_count(mock_accessible)
        assert result == expected_result
        if expects_debug_call:
            essential_modules["orca.debug"].print_tokens.assert_called()

    @pytest.mark.parametrize(
        "selected_count, index, expected_child",
        [
            pytest.param(3, 0, "child_0", id="first_child"),
            pytest.param(3, 2, "child_2", id="last_child"),
            pytest.param(3, -1, "child_2", id="negative_index_last"),
            pytest.param(0, 0, None, id="no_selected_children"),
            pytest.param(3, 5, None, id="index_out_of_bounds"),
            pytest.param(3, -2, None, id="invalid_negative_index"),
        ],
    )
    def test_get_selected_child(self, test_context, selected_count, index, expected_child) -> None:
        """Test AXSelection.get_selected_child."""

        mock_accessible = test_context.Mock(spec=Atspi.Accessible)
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.ax_selection import AXSelection

        mock_children = {i: f"child_{i}" for i in range(selected_count)}

        def mock_get_selected_child(_obj, idx):
            return mock_children.get(idx)

        test_context.patch_object(
            AXSelection,
            "get_selected_child_count",
            return_value=selected_count,
        )
        test_context.patch_object(
            Atspi.Selection,
            "get_selected_child",
            side_effect=mock_get_selected_child,
        )
        result = AXSelection.get_selected_child(mock_accessible, index)
        assert result == expected_child
        if expected_child is not None:
            essential_modules["orca.debug"].print_tokens.assert_called()

    @pytest.mark.parametrize(
        "error_scenario,expected_result,expects_debug_call",
        [
            pytest.param("returns_self", None, True, id="returns_self"),
            pytest.param("glib_error", None, True, id="glib_error"),
        ],
    )
    def test_get_selected_child_error_scenarios(
        self,
        test_context: OrcaTestContext,
        error_scenario: str,
        expected_result,
        expects_debug_call: bool,
    ) -> None:
        """Test AXSelection.get_selected_child error scenarios."""

        mock_accessible = test_context.Mock(spec=Atspi.Accessible)
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca import debug
        from orca.ax_selection import AXSelection

        test_context.patch_object(AXSelection, "get_selected_child_count", return_value=1)

        if error_scenario == "returns_self":
            test_context.patch_object(
                Atspi.Selection,
                "get_selected_child",
                return_value=mock_accessible,
            )
        elif error_scenario == "glib_error":

            def raise_glib_error(obj, idx):
                raise GLib.GError("Test error")

            test_context.patch_object(
                Atspi.Selection,
                "get_selected_child",
                side_effect=raise_glib_error,
            )

        test_context.patch_object(
            debug,
            "print_tokens",
            new=essential_modules["orca.debug"].print_tokens,
        )
        result = AXSelection.get_selected_child(mock_accessible, 0)
        assert result == expected_result
        if expects_debug_call:
            essential_modules["orca.debug"].print_tokens.assert_called()
